"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _pgSql = require("pg-sql2");

var _pgSql2 = _interopRequireDefault(_pgSql);

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debugSql = (0, _debug2.default)("graphile-build-pg:sql");

/*
   * Originally we tried this with a CTE, but:
   *
   * > The sub-statements in WITH are executed concurrently
   * > with each other and with the main query. Therefore, when
   * > using data-modifying statements in WITH, the order in
   * > which the specified updates actually happen is
   * > unpredictable. All the statements are executed with the
   * > same snapshot (see Chapter 13), so they cannot "see" one
   * > another's effects on the target tables. This alleviates
   * > the effects of the unpredictability of the actual order
   * > of row updates, and means that RETURNING data is the only
   * > way to communicate changes between different WITH
   * > sub-statements and the main query.
   *
   * -- https://www.postgresql.org/docs/9.6/static/queries-with.html
   *
   * This caused issues with computed columns that themselves
   * went off and performed selects - because the data within
   * those selects used the old snapshot and thus returned
   * stale data. To solve this, we now use temporary tables to
   * ensure the mutation and the select execute in different
   * statments.
   */

exports.default = async function viaTemporaryTable(pgClient, sqlTypeIdentifier, sqlMutationQuery, sqlResultSourceAlias, sqlResultQuery, isPgClassLike = true) {
  async function performQuery(pgClient, sqlQuery) {
    const { text, values } = _pgSql2.default.compile(sqlQuery);
    if (debugSql.enabled) debugSql(text);
    return pgClient.query(text, values);
  }

  if (!sqlTypeIdentifier) {
    // It returns void, just perform the query!
    return await performQuery(pgClient, _pgSql2.default.query`
      with ${sqlResultSourceAlias} as (
        ${sqlMutationQuery}
      ) ${sqlResultQuery}`);
  } else {
    const sqlTemporaryTableAlias = _pgSql2.default.identifier(`__temporary_${String(Math.random()).replace(/[^0-9]+/g, "")}__`);

    await performQuery(pgClient, _pgSql2.default.query`
      create temporary table ${sqlTemporaryTableAlias} (
        id serial not null,
        row ${sqlTypeIdentifier}
      ) on commit drop`);

    await performQuery(pgClient, _pgSql2.default.query`
      with ${sqlResultSourceAlias} as (
        ${sqlMutationQuery}
      )
      insert into ${sqlTemporaryTableAlias} (row)
      select ${isPgClassLike ? sqlResultSourceAlias : _pgSql2.default.query`${sqlResultSourceAlias}.${sqlResultSourceAlias}`}::${sqlTypeIdentifier} from ${sqlResultSourceAlias}`);
    const results = await performQuery(pgClient, _pgSql2.default.query`
      with ${sqlResultSourceAlias} as (
        select ${isPgClassLike ? _pgSql2.default.query`(${sqlTemporaryTableAlias}.row).*` : _pgSql2.default.query`${sqlTemporaryTableAlias}.row as ${sqlResultSourceAlias}`} from ${sqlTemporaryTableAlias} order by id asc
      )
      ${sqlResultQuery}`);
    await performQuery(pgClient, _pgSql2.default.query`drop table ${sqlTemporaryTableAlias};`);
    return results;
  }
};
//# sourceMappingURL=viaTemporaryTable.js.map